//This is the C++ implementation of the balanced network deconvolution (BND) algorithm, 
//which is demonstrated a general approach for removing transitive noise.

 //INPUT ARGUMENTS:
 //Mat a           Input matrix, which is a symetric matrix, is a relevance matrix where mat(i,j) represents the similarity content
 //              between nodes i and j. Elements of the input matrix are non-negative.
 
 //OUTPUT ARGUMENTS:
 //mat_bnd       Output balanced deconvolved matrix . Elements of the output matrix represent deconvolved direct edge weights of observed interactions.
 //              Usually top ranking weights can be used for application.
 
// Note: change #include </home/hpsun/Desktop/Node_new/PPI/eigen-eigen-6b38706d90a9/Eigen/Dense> to add local library;
//       change ifstream in("/home/hpsun/Desktop/Node_new/PPI/input_format.txt") to your local input file.

// Contact: hbshen@sjtu.edu.cn (Hong-Bin Shen), harrytobe@sjtu.edu.cn (Hai-Ping Sun)

//==========================================================================================

#include </home/hpsun/Desktop/Node_new/PPI/Eigen/Eigen/Dense>
#include <complex>
#include <iostream>
#include <sstream>
#include <vector>
#include <string>
#include <fstream>
using namespace std;
using namespace Eigen;

typedef vector<vector<double> > Mat;

void foo(MatrixXd& m)  
{  
    Matrix3d m2=Matrix3d::Zero(3,3);  
    m2(0,0)=1;  
    m=m2;  
}  

Mat input();
//main function
int main (void)
{
	   
	   //input matrix
	   Mat a = input();
	   int columns=a.size();
	   int m_size=columns;	  

	   //testing the input matrix
	   if (m_size==1)
	   	   cout<<"error: input matrix is empty"<<endl;
	   else
	   {
	   if (a.size()!=a[1].size())
		   cout<<"error: input matrix is not square"<<endl;
	   }
	   for (int i=0;i<m_size;i++)
	   {
		   for (int j=i+1;j<m_size;j++)
		   { 
			   if (a[i][j]!=a[j][i])
			   {
				   cout<<"error: input matrix is not square"<<endl;
				   break;
			   }
		   }
	   }

//
       MatrixXd m0(m_size,m_size);
	   for (int i=0;i<m_size;i++)
	   {
		   for (int j=0;j<m_size;j++)
			   m0(i,j)=0;
		//       m0(i,j)=a[i][j];
	   }
	   for (int i=0;i<m_size;i++)
	   {
		   for (int j=0;j<m_size;j++)
			   //m0(i,j)=0;
		       m0(i,j)=a[i][j];
	   }
	   cout<<"input matrix: "<<m0<<endl;

	   //processing the input matrix
	   //linearly mapping the input matrix to be between 0 and 1
	   double mmax=m0.maxCoeff();
	   double mmin=m0.minCoeff();
	   if (mmax!=mmin)
	   {
		   for (int i = 0; i<m_size;i++)
		   {
			    for(int j = 0; j<m_size;j++)
				{
					m0(i,j)=(m0(i,j)-mmin)/(mmax-mmin);
				}
		   }
	   }
	   else
		   cout<<"error: input matrix is a constant matrix"<<endl;

	   //filter diagonal values

	   if (mmax!=mmin)
	   {
		   for (int i = 0; i<m_size;i++)
		   {
			    for(int j = 0; j<m_size;j++)
				{
					if (i==j)
						m0(i,j)=0;
				}
		   }
	   }
	   else
		   cout<<"error: input matrix is a constant matrix"<<endl;
	   MatrixXd m1(m_size,m_size);
	   m1=m0;



	   //making the matrix symetric if already not
	   m1=(m1+m1.transpose())/2;

	   //eigen decomposition
	   //balanced network deconvolution
     EigenSolver<MatrixXd> es(m1);
	   double lamda_r;
	   MatrixXcd D=es.eigenvalues().asDiagonal();
	   //cout<<"Original D: "<<endl<<D<<endl;
	   MatrixXcd V=es.eigenvectors();
	   MatrixXcd mat_new;
	   for (int i = 0; i<m_size;i++)
	   {
		   if (abs(es.eigenvalues()[i])!=0)
			   {
				 lamda_r=real(es.eigenvalues()[i]);
			     lamda_r=(-1+sqrt(1+4*lamda_r*lamda_r))/2/lamda_r;
			     D(i,i)=lamda_r+0;
		       }
	   }

	   mat_new=V*D*V.inverse();
	   MatrixXd mat_newr(m_size,m_size);
	   for (int i = 0; i<m_size;i++)
		   for (int j = 0; j<m_size;j++)
			   mat_newr(i,j)=real(mat_new(i,j));

	   //display direct weights
	   MatrixXd mat_bnd(m_size,m_size);
	   MatrixXd ind_edges(m_size,m_size);
	   MatrixXd ind_nonedges(m_size,m_size);
	   for (int i = 0; i<m_size;i++)
		   for (int j = 0; j<m_size;j++)
			   if (m1(i,j)>0)
				   ind_edges(i,j)=1;
			   else
				   ind_edges(i,j)=0;
	   for (int i = 0; i<m_size;i++)
		   for (int j = 0; j<m_size;j++)
			   if (m1(i,j)==0)
				   ind_nonedges(i,j)=1;
			   else
				   ind_nonedges(i,j)=0;
	   MatrixXd ttemp(m_size,m_size);
	   for (int i = 0; i<m_size;i++)
		   for (int j = 0; j<m_size;j++)
			   ttemp(i,j)=m0(i,j)*ind_nonedges(i,j);
	   mmax=ttemp.maxCoeff();
	   mmin=mat_newr.minCoeff();
	   if (mmax-mmin>0)
		   for (int i = 0; i<m_size;i++)
		      for (int j = 0; j<m_size;j++)
			      mat_newr(i,j)=mat_newr(i,j)+mmax-mmin;
	   for (int i = 0; i<m_size;i++)
		   for (int j = 0; j<m_size;j++)
			   mat_bnd(i,j)=mat_newr(i,j)*ind_edges(i,j)+m0(i,j)*ind_nonedges(i,j);

	   //linearly mapping the deconvolved matrix to be between 0 and 1
	   mmax=mat_bnd.maxCoeff();
	   mmin=mat_bnd.minCoeff();
	   if (mmin!=mmax)
	   {
	      for (int i = 0; i<m_size;i++)
		      for (int j = 0; j<m_size;j++)
			      mat_bnd(i,j)=(mat_bnd(i,j)-mmin)/(mmax-mmin);
	   }
	   cout<<endl<<"BND matrix:"<<endl<<mat_bnd<<endl;
	   system("pause");
       return 0;
}

//read matrix function
Mat input()
{
       ifstream in("/home/hpsun/Desktop/Node_new/PPI/Input_format.txt");
       Mat a;
       istringstream istr;
       string str;
       vector<double> tmpvec;
       while(getline(in,str))
      {
           istr.str(str);
           double tmp;
           while(istr>>tmp)
          {
               tmpvec.push_back(tmp);
          }
         a.push_back(tmpvec);
         tmpvec.clear();
         istr.clear();
      }
    in.close();
   return a;
}
